@@ -4,7 +4,7 @@
  *  Written by:		Ullrich Hafner
  *		
  *  This file is part of FIASCO (Fractal Image And Sequence COdec)
- *  Copyright (C) 1994-2000 Ullrich Hafner <hafner@bigfoot.de>
+ *  Copyright (C) 1994-2000 Ullrich Hafner
  */
 
 /*
@@ -20,10 +20,7 @@
 #include "config.h"
 
 #include <string.h>
-#if STDC_HEADERS
-#	include <stdlib.h>
-#endif /* not STDC_HEADERS */
-
+#include <stdlib.h>
 #include <stdio.h>
 
 #include "nstring.h"
